/********************************************************************/
/*                                                                  */
/*      Provides rhe following code:                                */
/*        IRQ Handling                                              */
/*        MMR definitions                                           */
/*        newlib bindings                                           */
/*                                                                  */
/*		(C) Tilo Lutz, TiloLutz@gmx.de								*/
/*																	*/
/*		09.07.2008	Initial release                                 */
/*		12.07.2008	Added support for other aduc types              */
/*					The correct model must be set as symbol now.	*/
/*																	*/
/********************************************************************/

#ifndef ADUC_H_
#define ADUC_H_

//#define ADUC7026
#define USENEWLIB

extern void  	(*IRQ)(void);		/* 	Function Pointers for Interupts 	*/
extern void 	(*SWI)(void);
extern void 	(*FIQ)(void);
extern void  	(*UNDEF)(void);
extern void 	(*PABORT)(void);
extern void 	(*DABORT)(void);

/* Check MCU type is set */
#ifndef ADUC7019
#ifndef ADUC7020
#ifndef ADUC7021
#ifndef ADUC7022
#ifndef ADUC7023
#ifndef ADUC7024
#ifndef ADUC7025
#ifndef ADUC7026
#ifndef ADUC7027
#ifndef ADUC7028
#ifndef ADUC7032
#ifndef ADUC7033
#ifndef ADUC7034
#ifndef ADUC7128
#ifndef ADUC7129
#error Please define MCU type with -DADUC7xxx, where xxx represents the exact model type.
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif

#ifdef USENEWLIB
	extern char(*STDIN)(void);
	extern void (*STDOUT)(char ptr);
#endif

/*  INTERRUPT CONTROLLER    */
#define FIQ_SOURCE_BIT 		0x00000001						
#define SWI_BIT 			0x00000002						
#define RTOS_TIMER_BIT 		0x00000004						
#define GP_TIMER_BIT 		0x00000008						
#define WAKEUP_TIMER_BIT 	0x00000010						
#define WATCHDOG_TIMER_BIT 	0x00000020
	
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)

	#define FLASHCON_BIT 		0x00000040						
	#define ADC_BIT 			0x00000080						
	#define PLL_LOCK_BIT 		0x00000100						
	#define SM_SLAVE_BIT 		0x00000200						
	#define SM_MASTER0_BIT 		0x00000400						
	#define SM_MASTER1_BIT 		0x00000800						
	#define SPI_SLAVE_BIT 		0x00001000						
	#define SPI_MASTER_BIT 		0x00002000						
	#define UART_BIT 			0x00004000						
	#define XIRQ0_BIT 			0x00008000						
	#define MONITOR_BIT 		0x00010000						
	#define PSM_BIT 			0x00020000						
	#define XIRQ1_BIT 			0x00040000						
	#define PLA_IRQ0_BIT 		0x00080000						
	#define PLA_IRQ1_BIT 		0x00100000						
	#define SPM4_IO_BIT 		0x00200000						
	#define SPM5_IO_BIT 		0x00400000						
	#define PWM_BIT 			0x00800000									
#endif
#if defined (ADUC7128) || defined (ADUC7129)
	#define STI_TIMER_BIT 		0x00000040						
	#define FLASHCON0_BIT		0x00000080						
	#define FLASHCON1_BIT 		0x00000100						
	#define ADC_BIT	 			0x00000200						
	#define QEN_BIT 			0x00000400						
	#define SM_SLAVE0_BIT 		0x00000800						
	#define SM_SLAVE1_BIT 		0x00001000						
	#define SM_MASTER0_BIT 		0x00002000						
	#define SM_MASTER1_BIT		0x00004000						
	#define SPI_SLAVE_BIT 		0x00008000						
	#define SPI_MASTER_BIT 		0x00010000						
	#define UART0_BIT 			0x00020000						
	#define UART1_BIT 			0x00040000						
	#define XIRQ0_BIT 			0x00080000						
	#define MONITOR_BIT 		0x00100000						
	#define PSM_BIT 			0x00200000						
	#define XIRQ1_BIT 			0x00400000						
	#define PLA_IRQ0_BIT		0x00800000									
	#define PLA_IRQ1_BIT 		0x01000000						
	#define XIRQ2_BIT 			0x02000000						
	#define XIRQ3_BIT 			0x04000000						
	#define PWM_BIT 			0x08000000						
	#define PLL_LOCK_BIT		0x10000000									
#endif
#if defined (ADUC7033) || defined (ADUC7034)
	#define STI_TIMER_BIT 		0x00000040						
#endif
#if defined (ADUC7032) || defined (ADUC7033) || defined (ADUC7034)
	#define LIN_BIT				0x00000080						
	#define FLASHCON_BIT 		0x00000100						
	#define PLL_LOCK_BIT		0x00000200						
	#define ADC_BIT 			0x00000400						
	#define UART_BIT 			0x00000800						
	#define SPI_MASTER_BIT 		0x00001000						
	#define XIRQ0_BIT 			0x00002000						
	#define XIRQ1_BIT			0x00004000						
	#define IRQ3_BIT 			0x00010000						
	#define SPI_SLAVE_BIT 		0x00020000						
	#define XIRQ3_BIT 			0x00040000						
	#define XIRQ4_BIT 			0x00080000						
#endif
	
                                        
#define IRQSTA 		(*(volatile unsigned long *) 0XFFFF0000)
#define IRQSIG 		(*(volatile unsigned long *) 0XFFFF0004)
#define IRQEN 		(*(volatile unsigned long *) 0XFFFF0008)
#define IRQCLR 		(*(volatile unsigned long *) 0XFFFF000C)
#define SWICFG 		(*(volatile unsigned long *) 0XFFFF0010)
#define FIQSTA 		(*(volatile unsigned long *) 0XFFFF0100)
#define FIQSIG 		(*(volatile unsigned long *) 0XFFFF0104)
#define FIQEN 		(*(volatile unsigned long *) 0XFFFF0108)
#define FIQCLR 		(*(volatile unsigned long *) 0XFFFF010C)


/*  REMAP AND SYSTEM CONTROL  */
#define RSTSTA 		(*(volatile unsigned char *) 	0XFFFF0230)
#define RSTCLR 		(*(volatile unsigned char *) 	0XFFFF0234)
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define REMAP 		(*(volatile unsigned char *) 	0XFFFF0220)
#endif
#if defined (ADUC7023) || defined (ADUC7033) || defined (ADUC7034)
	#define SYSMAP0		(*(volatile unsigned char *) 	0XFFFF0220)
	#define SYSSER0		(*(volatile unsigned long *) 	0XFFFF0238)
	#define SYSSER1		(*(volatile unsigned long *) 	0XFFFF023C)
	#define SYSALI		(*(volatile unsigned long *) 	0XFFFF0560)
	#define SYSCHK		(*(volatile unsigned long *) 	0XFFFF0240)
#endif
	
/*  TIMER 0    */
#define T0LD 		(*(volatile unsigned short *)	0XFFFF0300)
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)

	#define T0VAL 		(*(volatile unsigned short *) 	0XFFFF0304)
	#define T0CON 		(*(volatile unsigned long *)	0XFFFF0308)
	#define T0CLRI 		(*(volatile unsigned char *) 	0XFFFF030C)
#endif
#if defined (ADUC7032) || defined (ADUC7033) || defined (ADUC7034) || defined (ADUC7128) || defined (ADUC7129)
	#define T0VAL0 		(*(volatile unsigned short *) 	0XFFFF0304)
	#define T0VAL1 		(*(volatile unsigned long *) 	0XFFFF0308)
	#define T0CON 		(*(volatile unsigned long *)	0XFFFF030C)
	#define T0ICLR 		(*(volatile unsigned char *) 	0XFFFF0310)
	#define T0CLRI 		(*(volatile unsigned char *) 	0XFFFF0310)
	#define T0CAP 		(*(volatile unsigned short *) 	0XFFFF0314)
#endif
	
/*  GENERAL PURPOSE TIMER 0  */
#define T1LD 		(*(volatile unsigned long *) 	0XFFFF0320)
#define T1VAL 		(*(volatile unsigned long *) 	0XFFFF0324)
#define T1CON 		(*(volatile unsigned long *) 	0XFFFF0328)
#define T1CLRI 		(*(volatile unsigned char *) 	0XFFFF032C)
#define T1ICLR 		(*(volatile unsigned char *) 	0XFFFF032C)
#define T1CAP 		(*(volatile unsigned long *) 	0XFFFF0330)

/*  WAKE UP TIMER   */
#define T2LD 		(*(volatile unsigned long *) 	0XFFFF0340)
#define T2VAL 		(*(volatile unsigned long *) 	0XFFFF0344)
#define T2CON 		(*(volatile unsigned long *) 	0XFFFF0348)
#define T2CLRI 		(*(volatile unsigned char *) 	0XFFFF034C)
#define T2ICLR 		(*(volatile unsigned char *) 	0XFFFF034C)

/*  WATCHDOG TIMER    */
#define T3LD 		(*(volatile unsigned short *) 	0XFFFF0360)
#define T3VAL 		(*(volatile unsigned short *) 	0XFFFF0364)
#define T3CON 		(*(volatile unsigned short *) 	0XFFFF0368)
#define T3CLRI 		(*(volatile unsigned char *) 	0XFFFF036C)
#define T3ICLR		(*(volatile unsigned char *) 	0XFFFF036C)

#if defined (ADUC7034) || defined (ADUC7128) || defined (ADUC7129)
	/*  STI TIMER  */
	#define T4LD 		(*(volatile unsigned long *) 	0XFFFF0380)
	#define T4VAL 		(*(volatile unsigned long *) 	0XFFFF0384)
	#define T4CON 		(*(volatile unsigned long *) 	0XFFFF0388)
	#define T4CLRI 		(*(volatile unsigned char *) 	0XFFFF038C)
	#define T4CAP 		(*(volatile unsigned long *) 	0XFFFF0390)
#endif

/*  PLL AND OSCILLATOR CONTROL  */
#define POWSTA		(*(volatile unsigned char *) 	0XFFFF0400)
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define POWKEY1		(*(volatile unsigned short *) 	0XFFFF0404)
	#define POWCON 		(*(volatile unsigned short *) 	0XFFFF0408)
	#define POWKEY2		(*(volatile unsigned short *) 	0XFFFF040C)
	#define PLLKEY1		(*(volatile unsigned short *) 	0XFFFF0410)
	#define PLLCON 		(*(volatile unsigned short *) 	0XFFFF0414)
	#define PLLKEY2		(*(volatile unsigned short *) 	0XFFFF0418)
#endif
#if defined (ADUC7032) || defined (ADUC7033) || defined (ADUC7034)
	#define POWKEY0		(*(volatile unsigned long *) 	0XFFFF0404)
	#define POWCON		(*(volatile unsigned char *) 	0XFFFF0408)
	#define POWKEY1		(*(volatile unsigned long *) 	0XFFFF040C)
	#define PLLKEY0		(*(volatile unsigned long *) 	0XFFFF0410)
	#define PLLCON		(*(volatile unsigned char *) 	0XFFFF0414)
	#define PLLKEY1		(*(volatile unsigned long *) 	0XFFFF0418)
	#define OSC0TRM		(*(volatile unsigned char *) 	0XFFFF042C)
	#define OSC0CON		(*(volatile unsigned char *) 	0XFFFF0440)
	#define OSC0STA		(*(volatile unsigned char *) 	0XFFFF0444)
	#define OSC0VAL0	(*(volatile unsigned short *) 	0XFFFF0448)
	#define OSC0VAL1	(*(volatile unsigned short *) 	0XFFFF044C)
#endif
	
/*  POWER SUPPLY MONITOR   */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define PSMCON 		(*(volatile unsigned short *) 	0XFFFF0440)
	#define CMPCON 		(*(volatile unsigned short *) 	0XFFFF0444)
#endif
	
/*	Band Gap Reference */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define REFCON 		(*(volatile unsigned char *) 	0XFFFF048C)
#endif
	
/*  ADC INTERFACE REGISTERS   */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define ADCCON 		(*(volatile unsigned short *) 	0XFFFF0500)
	#define ADCCP 		(*(volatile unsigned char *) 	0XFFFF0504)
	#define ADCCN 		(*(volatile unsigned char *) 	0XFFFF0508)
	#define ADCSTA 		(*(volatile unsigned char *) 	0XFFFF050C)
	#define ADCDAT 		(*(volatile unsigned long *) 	0XFFFF0510)
	#define ADCRST 		(*(volatile unsigned char *) 	0XFFFF0514)
#endif
	
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)
	
	#define ADCGN 		(*(volatile unsigned short *) 	0XFFFF0530)
	#define ADCOF 		(*(volatile unsigned short *) 	0XFFFF0534)
#endif
	
#if defined (ADUC7023) || defined (ADUC7033) || defined (ADUC7034)
	#define ADCSTA 		(*(volatile unsigned short *) 	0XFFFF0500)
	#define ADCMSKI		(*(volatile unsigned char *) 	0XFFFF0504)
	#define ADCMDE 		(*(volatile unsigned char *) 	0XFFFF0508)
	#define ADC0CON		(*(volatile unsigned short *) 	0XFFFF050C)
	#define ADC1CON		(*(volatile unsigned short *) 	0XFFFF0510)
	#define ADCFLT 		(*(volatile unsigned short *) 	0XFFFF0518)
	#define ADCCFG 		(*(volatile unsigned char *) 	0XFFFF051C)
	#define ADC0DAT		(*(volatile unsigned short *) 	0XFFFF0520)
	#define ADC1DAT		(*(volatile unsigned short *) 	0XFFFF0524)
	#define ADC2DAT		(*(volatile unsigned short *) 	0XFFFF0528)
	#define ADC0OF 		(*(volatile unsigned short *) 	0XFFFF0530)
	#define ADC1OF 		(*(volatile unsigned short *) 	0XFFFF0534)
	#define ADC2OF 		(*(volatile unsigned short *) 	0XFFFF0538)
	#define ADC0GN 		(*(volatile unsigned short *) 	0XFFFF053C)
	#define ADC1GN 		(*(volatile unsigned short *) 	0XFFFF0540)
	#define ADC2GN 		(*(volatile unsigned short *) 	0XFFFF0544)
	#define ADC0RCL		(*(volatile unsigned short *) 	0XFFFF0548)
	#define ADC0RCV		(*(volatile unsigned short *) 	0XFFFF054C)
	#define ADC0TH 		(*(volatile unsigned short *) 	0XFFFF0550)
	#define ADC0TCL		(*(volatile unsigned char *) 	0XFFFF0554)
	#define ADC0THV		(*(volatile unsigned char *) 	0XFFFF0558)
	#define ADC0ACC		(*(volatile unsigned long *) 	0XFFFF055C)
	#define ADCREF 		(*(volatile unsigned short *) 	0XFFFF057C)
#endif
	
/* EXTERNAL MEMORY REGISTERS */
#if defined (ADUC7026) || (ADUC7027) || defined (ADUC7028)
	#define XMCFG		(*(volatile unsigned char *)	0XFFFFF000)
	#define XM0CON		(*(volatile unsigned char *)	0XFFFFF010)
	#define XM1CON		(*(volatile unsigned char *)	0XFFFFF014)
	#define XM2CON		(*(volatile unsigned char *)	0XFFFFF018)
	#define XM3CON		(*(volatile unsigned char *)	0XFFFFF01C)
	#define XM0PAR		(*(volatile unsigned short *)	0XFFFFF020)
	#define XM1PAR		(*(volatile unsigned short *)	0XFFFFF024)
	#define XM2PAR		(*(volatile unsigned short *)	0XFFFFF028)
	#define XM3PAR		(*(volatile unsigned short *)	0XFFFFF02C)
#endif
#if defined (ADUC7129)
	#define XMCFG		(*(volatile unsigned char *)	0XFFFF0C00)
	#define XM0CON		(*(volatile unsigned char *)	0XFFFF0C10)
	#define XM1CON		(*(volatile unsigned char *)	0XFFFF0C14)
	#define XM2CON		(*(volatile unsigned char *)	0XFFFF0C18)
	#define XM3CON		(*(volatile unsigned char *)	0XFFFF0C1C)
	#define XM0PAR		(*(volatile unsigned short *)	0XFFFF0C20)
	#define XM1PAR		(*(volatile unsigned short *)	0XFFFF0C24)
	#define XM2PAR		(*(volatile unsigned short *)	0XFFFF0C28)
	#define XM3PAR		(*(volatile unsigned short *)	0XFFFF0C2C)
#endif
	
/*  DAC INTERFACE REGISTERS   */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7023) \
	|| defined (ADUC7024)  || defined (ADUC7026) || defined (ADUC7028)

	#define DAC0CON 	(*(volatile unsigned char *)	0XFFFF0600)
	#define DAC0DAT 	(*(volatile unsigned long *) 	0XFFFF0604)
	#define DAC1CON 	(*(volatile unsigned char *) 	0XFFFF0608)
	#define DAC1DAT 	(*(volatile unsigned long *) 	0XFFFF060C)

	#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7026) || defined (ADUC7028)
		#define DAC2CON 	(*(volatile unsigned char *) 	0XFFFF0610)
		#define DAC2DAT 	(*(volatile unsigned long *) 	0XFFFF0614)
		#if defined (ADUC7020) || defined (ADUC7026) || defined (ADUC7028)
			#define DAC3CON 	(*(volatile unsigned char *) 	0XFFFF0618)
			#define DAC3DAT 	(*(volatile unsigned long *) 	0XFFFF061C)
		#endif
	#endif
#endif
	
/* DDS INTERFACE REGISTERS ADUC712x */
#if defined (ADUC7128) || defined (ADUC7129)
	#define DACCON 		(*(volatile unsigned short *)	0XFFFF0670)
	#define DDSCON 		(*(volatile unsigned char *)	0XFFFF0690)
	#define DDSFRQ 		(*(volatile unsigned long *)	0XFFFF0694)
	#define DDSPHS 		(*(volatile unsigned short *)	0XFFFF0698)
	#define DACKEY0		(*(volatile unsigned char *)	0XFFFF06A4)
	#define DACDAT 		(*(volatile unsigned short *)	0XFFFF06B4)
	#define DACEN 		(*(volatile unsigned char *)	0XFFFF06B8)
	#define DACKEY1		(*(volatile unsigned char *)	0XFFFF06BC)
#endif
	
/*  450 COMPATIABLE UART CORE REGISTERS DEVICE 1*/
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7032) || defined (ADUC7033) \
	|| defined (ADUC7034)
	
	#define COMTX 	 	(*(volatile unsigned char *) 	0XFFFF0700)
	#define COMRX 	 	(*(volatile unsigned char *) 	0XFFFF0700)
	#define COMDIV0  	(*(volatile unsigned char *) 	0XFFFF0700)
	#define COMIEN0  	(*(volatile unsigned char *) 	0XFFFF0704)
	#define COMDIV1  	(*(volatile unsigned char *) 	0XFFFF0704)
	#define COMIID0  	(*(volatile unsigned char *) 	0XFFFF0708)
	#define COMCON0  	(*(volatile unsigned char *) 	0XFFFF070C)
	#define COMCON1  	(*(volatile unsigned char *) 	0XFFFF0710)
	#define COMSTA0  	(*(volatile unsigned char *) 	0XFFFF0714)
	#define COMSTA1  	(*(volatile unsigned char *) 	0XFFFF0718)
	#define COMSCR   	(*(volatile unsigned char *) 	0XFFFF071C)
	#define COMIEN1  	(*(volatile unsigned char *) 	0XFFFF0720)
	#define COMIID1  	(*(volatile unsigned char *) 	0XFFFF0724)
	#define COMADR 	 	(*(volatile unsigned char *) 	0XFFFF0728)
	#define COMDIV2  	(*(volatile unsigned short *) 	0XFFFF072C)
#endif

#if defined (ADUC7128) || defined (ADUC7129)
	#define COM0TX 	 	(*(volatile unsigned char *) 	0XFFFF0700)
	#define COM0RX 	 	(*(volatile unsigned char *) 	0XFFFF0700)
	#define COM0DIV0  	(*(volatile unsigned char *) 	0XFFFF0700)
	#define COM0IEN0  	(*(volatile unsigned char *) 	0XFFFF0704)
	#define COM0DIV1  	(*(volatile unsigned char *) 	0XFFFF0704)
	#define COM0IID0  	(*(volatile unsigned char *) 	0XFFFF0708)
	#define COM0CON0  	(*(volatile unsigned char *) 	0XFFFF070C)
	#define COM0CON1  	(*(volatile unsigned char *) 	0XFFFF0710)
	#define COM0STA0  	(*(volatile unsigned char *) 	0XFFFF0714)
	#define COM0STA1  	(*(volatile unsigned char *) 	0XFFFF0718)
	#define COM0SCR   	(*(volatile unsigned char *) 	0XFFFF071C)
	#define COM0IEN1  	(*(volatile unsigned char *) 	0XFFFF0720)
	#define COM0IID1  	(*(volatile unsigned char *) 	0XFFFF0724)
	#define COM0ADR  	(*(volatile unsigned char *) 	0XFFFF0728)
	#define COM0DIV2  	(*(volatile unsigned short *) 	0XFFFF072C)

	/*  450 COMPATIABLE UART CORE REGISTERS DEVICE 2 */
	#define COM1TX 	 	(*(volatile unsigned char *) 	0XFFFF0740)
	#define COM1RX 	 	(*(volatile unsigned char *) 	0XFFFF0740)
	#define COM1DIV0  	(*(volatile unsigned char *) 	0XFFFF0740)
	#define COM1IEN0  	(*(volatile unsigned char *) 	0XFFFF0744)
	#define COM1DIV1  	(*(volatile unsigned char *) 	0XFFFF0744)
	#define COM1IID0  	(*(volatile unsigned char *) 	0XFFFF0748)
	#define COM1CON0  	(*(volatile unsigned char *) 	0XFFFF074C)
	#define COM1CON1  	(*(volatile unsigned char *) 	0XFFFF0750)
	#define COM1STA0  	(*(volatile unsigned char *) 	0XFFFF0754)
	#define COM1STA1  	(*(volatile unsigned char *) 	0XFFFF0758)
	#define COM1SCR   	(*(volatile unsigned char *) 	0XFFFF075C)
	#define COM1IEN1  	(*(volatile unsigned char *) 	0XFFFF0760)
	#define COM1IID1  	(*(volatile unsigned char *) 	0XFFFF0764)
	#define COM1ADR	 	(*(volatile unsigned char *) 	0XFFFF0768)
	#define COM1DIV2  	(*(volatile unsigned short *) 	0XFFFF076C)
#endif
	
/*  I2C BUS PERIPHERAL DEVICE 1 */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define I2C0MSTA 	(*(volatile unsigned char *) 	0XFFFF0800)
	#define I2C0SSTA 	(*(volatile unsigned char *) 	0XFFFF0804)
	#define I2C0SRX 	(*(volatile unsigned char *) 	0XFFFF0808)
	#define I2C0STX 	(*(volatile unsigned char *) 	0XFFFF080C)
	#define I2C0MRX 	(*(volatile unsigned char *) 	0XFFFF0810)
	#define I2C0MTX 	(*(volatile unsigned char *) 	0XFFFF0814)
	#define I2C0CNT 	(*(volatile unsigned char *) 	0XFFFF0818)
	#define I2C0ADR 	(*(volatile unsigned char *) 	0XFFFF081C)
	#define I2C0ALT 	(*(volatile unsigned char *)	0XFFFF0828)
	#define I2C0CFG 	(*(volatile unsigned char *) 	0XFFFF082C)
	#define I2C0DIV	 	(*(volatile unsigned short *) 	0XFFFF0830)
	#define I2C0DIVL 	(*(volatile unsigned char *) 	0XFFFF0830)
	#define I2C0DIVH 	(*(volatile unsigned char *) 	0XFFFF0831)
	#define I2C0ID0 	(*(volatile unsigned char *) 	0XFFFF0838)
	#define I2C0ID1 	(*(volatile unsigned char *) 	0XFFFF083C)
	#define I2C0ID2 	(*(volatile unsigned char *) 	0XFFFF0840)
	#define I2C0ID3 	(*(volatile unsigned char *) 	0XFFFF0844)
#endif
	
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)
	
	#define I2C0BYTE 	(*(volatile unsigned char *) 	0XFFFF0824)
	#define I2C0CCNT 	(*(volatile unsigned char *) 	0XFFFF0848)
	#define I2C0FSTA 	(*(volatile unsigned short *) 	0XFFFF084C)
#endif
#if defined (ADUC7128) || defined (ADUC7129)
	#define I2C0BYT 	(*(volatile unsigned char *) 	0XFFFF0824)
	#define I2C0SSC 	(*(volatile unsigned char *) 	0XFFFF0848)
	#define I2C0FIF 	(*(volatile unsigned char *) 	0XFFFF084C)
#endif

/*  I2C BUS PERIPHERAL DEVICE 2 */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define I2C1MSTA 	(*(volatile unsigned char *) 	0XFFFF0900)
	#define I2C1SSTA 	(*(volatile unsigned char *) 	0XFFFF0904)
	#define I2C1SRX 	(*(volatile unsigned char *)	0XFFFF0908)
	#define I2C1STX 	(*(volatile unsigned char *) 	0XFFFF090C)
	#define I2C1MRX 	(*(volatile unsigned char *) 	0XFFFF0910)
	#define I2C1MTX 	(*(volatile unsigned char *) 	0XFFFF0914)
	#define I2C1CNT 	(*(volatile unsigned char *) 	0XFFFF0918)
	#define I2C1ADR 	(*(volatile unsigned char *) 	0XFFFF091C)
	#define I2C1ALT 	(*(volatile unsigned char *) 	0XFFFF0928)
	#define I2C1CFG 	(*(volatile unsigned char *) 	0XFFFF092C)
	#define I2C1DIV	 	(*(volatile unsigned short *) 	0XFFFF0930)
	#define I2C1DIVL 	(*(volatile unsigned char *) 	0XFFFF0930)
	#define I2C1DIVH 	(*(volatile unsigned char *) 	0XFFFF0931)
	#define I2C1ID0 	(*(volatile unsigned char *) 	0XFFFF0938)
	#define I2C1ID1 	(*(volatile unsigned char *) 	0XFFFF093C)
	#define I2C1ID2 	(*(volatile unsigned char *) 	0XFFFF0940)
	#define I2C1ID3 	(*(volatile unsigned char *) 	0XFFFF0944)
#endif
	
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)
	
	#define I2C1BYTE 	(*(volatile unsigned char *) 	0XFFFF0924)
	#define I2C1CCNT 	(*(volatile unsigned char *) 	0XFFFF0948)
	#define I2C1FSTA 	(*(volatile unsigned short *) 	0XFFFF094C)
#endif
#if defined (ADUC7128) || defined (ADUC7129)
	#define I2C1BYT 	(*(volatile unsigned char *) 	0XFFFF0924)
	#define I2C1SSC 	(*(volatile unsigned char *) 	0XFFFF0948)
	#define I2C1FIF 	(*(volatile unsigned char *) 	0XFFFF094C)
#endif

/*  SERIAL PORT INTERFACE PERIPHERAL  */
#define SPISTA  	(*(volatile unsigned char *) 	0XFFFF0A00)
#define SPIRX 		(*(volatile unsigned char *) 	0XFFFF0A04)
#define SPITX 		(*(volatile unsigned char *) 	0XFFFF0A08)
#define SPIDIV 		(*(volatile unsigned char *) 	0XFFFF0A0C)
#define SPICON 		(*(volatile unsigned short *) 	0XFFFF0A10)

/*  PROGRAMABLE LOGIC ARRAY   */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028) || defined (ADUC7128) || defined (ADUC7129)

	#define PLAELM0 	(*(volatile unsigned short *) 	0XFFFF0B00)
	#define PLAELM1 	(*(volatile unsigned short *) 	0XFFFF0B04)
	#define PLAELM2 	(*(volatile unsigned short *) 	0XFFFF0B08)
	#define PLAELM3 	(*(volatile unsigned short *) 	0XFFFF0B0C)
	#define PLAELM4 	(*(volatile unsigned short *)	0XFFFF0B10)
	#define PLAELM5 	(*(volatile unsigned short *) 	0XFFFF0B14)
	#define PLAELM6 	(*(volatile unsigned short *) 	0XFFFF0B18)
	#define PLAELM7 	(*(volatile unsigned short *) 	0XFFFF0B1C)
	#define PLAELM8 	(*(volatile unsigned short *) 	0XFFFF0B20)
	#define PLAELM9 	(*(volatile unsigned short *) 	0XFFFF0B24)
	#define PLAELM10 	(*(volatile unsigned short *) 	0XFFFF0B28)
	#define PLAELM11 	(*(volatile unsigned short *) 	0XFFFF0B2C)
	#define PLAELM12 	(*(volatile unsigned short *) 	0XFFFF0B30)
	#define PLAELM13 	(*(volatile unsigned short *) 	0XFFFF0B34)
	#define PLAELM14 	(*(volatile unsigned short *) 	0XFFFF0B38)
	#define PLAELM15 	(*(volatile unsigned short *) 	0XFFFF0B3C)
	#define PLACLK 		(*(volatile unsigned char *) 	0XFFFF0B40)
	#define PLAIRQ 		(*(volatile unsigned long *) 	0XFFFF0B44)
	#define PLAADC 		(*(volatile unsigned long *) 	0XFFFF0B48)
	#define PLADIN 		(*(volatile unsigned long *) 	0XFFFF0B4C)
	#define PLADOUT 	(*(volatile unsigned long *) 	0XFFFF0B50)
	#define PLALCK	 	(*(volatile unsigned long *) 	0XFFFF0B54)
#endif
	
/*  GPIO AND SERIAL PORT MUX */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)

	#define GP0CON 		(*(volatile unsigned long *) 	0XFFFFF400)
	#define GP0DAT 		(*(volatile unsigned long *) 	0XFFFFF420)
	#define GP0SET 		(*(volatile unsigned long *) 	0XFFFFF424)
	#define GP0CLR 		(*(volatile unsigned long *) 	0XFFFFF428)
	#define GP0PAR 		(*(volatile unsigned long *) 	0XFFFFF42C)
	#define GP1CON 		(*(volatile unsigned long *) 	0XFFFFF404)
	#define GP1DAT 		(*(volatile unsigned long *) 	0XFFFFF430)
	#define GP1SET 		(*(volatile unsigned long *) 	0XFFFFF434)
	#define GP1CLR 		(*(volatile unsigned long *) 	0XFFFFF438)
	#define GP1PAR 		(*(volatile unsigned long *) 	0XFFFFF43C)
	#define GP2CON 		(*(volatile unsigned long *) 	0XFFFFF408)
	#define GP2DAT 		(*(volatile unsigned long *) 	0XFFFFF440)
	#define GP2SET 		(*(volatile unsigned long *) 	0XFFFFF444)
	#define GP2CLR 		(*(volatile unsigned long *) 	0XFFFFF448)
	#define GP3CON 		(*(volatile unsigned long *) 	0XFFFFF40C)
	#define GP3DAT 		(*(volatile unsigned long *) 	0XFFFFF450)
	#define GP3SET 		(*(volatile unsigned long *) 	0XFFFFF454)
	#define GP3CLR 		(*(volatile unsigned long *) 	0XFFFFF458)
	#define GP3PAR 		(*(volatile unsigned long *) 	0XFFFFF45C)
	#define GP4CON 		(*(volatile unsigned long *) 	0XFFFFF410)
	#define GP4DAT 		(*(volatile unsigned long *) 	0XFFFFF460)
	#define GP4SET 		(*(volatile unsigned long *) 	0XFFFFF464)
	#define GP4CLR 		(*(volatile unsigned long *) 	0XFFFFF468)
#endif
#if defined (ADUC7032) || defined (ADUC7033) || defined (ADUC7034) || defined (ADUC7128) || defined (ADUC7129)
	#define GP0CON 		(*(volatile unsigned long *) 	0XFFFF0D00)
	#define GP0DAT 		(*(volatile unsigned long *) 	0XFFFF0D20)
	#define GP0SET 		(*(volatile unsigned long *) 	0XFFFF0D24)
	#define GP0CLR 		(*(volatile unsigned long *) 	0XFFFF0D28)
	#define GP0PAR 		(*(volatile unsigned long *) 	0XFFFF0D2C)
	#define GP1CON 		(*(volatile unsigned long *) 	0XFFFF0D04)
	#define GP1DAT 		(*(volatile unsigned long *) 	0XFFFF0D30)
	#define GP1SET 		(*(volatile unsigned long *) 	0XFFFF0D34)
	#define GP1CLR 		(*(volatile unsigned long *) 	0XFFFF0D38)
	#define GP1PAR 		(*(volatile unsigned long *) 	0XFFFF0D3C)
	#define GP2CON 		(*(volatile unsigned long *) 	0XFFFF0D08)
	#define GP2DAT 		(*(volatile unsigned long *) 	0XFFFF0D40)
	#define GP2SET 		(*(volatile unsigned long *) 	0XFFFF0D44)
	#define GP2CLR 		(*(volatile unsigned long *) 	0XFFFF0D48)
#endif	
#if defined (ADUC7128) || defined (ADUC7129)
	#define GP3CON 		(*(volatile unsigned long *) 	0XFFFF0D0C)
	#define GP3DAT 		(*(volatile unsigned long *) 	0XFFFF0D50)
	#define GP3SET 		(*(volatile unsigned long *) 	0XFFFF0D54)
	#define GP3CLR 		(*(volatile unsigned long *) 	0XFFFF0D58)
	#define GP3PAR 		(*(volatile unsigned long *) 	0XFFFF0D5C)
	#define GP4CON 		(*(volatile unsigned long *) 	0XFFFF0D10)
	#define GP4DAT 		(*(volatile unsigned long *) 	0XFFFF0D60)
	#define GP4SET 		(*(volatile unsigned long *) 	0XFFFF0D64)
	#define GP4CLR 		(*(volatile unsigned long *) 	0XFFFF0D68)
#endif
	
/*  FLASH CONTROL INTERFACE   */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)

	#define FEESTA 		(*(volatile unsigned char *) 	0XFFFFF800)
	#define FEEMOD 		(*(volatile unsigned short *)	0XFFFFF804)
	#define FEECON 		(*(volatile unsigned char *) 	0XFFFFF808)
	#define FEEDAT 		(*(volatile unsigned short *) 	0XFFFFF80C)
	#define FEEADR 		(*(volatile unsigned short *) 	0XFFFFF810)
	#define FEESIGN 	(*(volatile unsigned long *) 	0XFFFFF818)
	#define FEEPRO 		(*(volatile unsigned long *) 	0XFFFFF81C)
	#define FEEHIDE 	(*(volatile unsigned long *) 	0XFFFFF820)
#endif
#if defined (ADUC7032) || defined (ADUC7033) || defined (ADUC7034) || defined (ADUC7128) || defined (ADUC7129)
	#define FEE0STA 		(*(volatile unsigned char *) 	0XFFFF0E00)
	#define FEE0MOD 		(*(volatile unsigned char *)	0XFFFF0E04)
	#define FEE0CON 		(*(volatile unsigned char *) 	0XFFFF0E08)
	#define FEE0DAT 		(*(volatile unsigned short *) 	0XFFFF0E0C)
	#define FEE0ADR 		(*(volatile unsigned short *) 	0XFFFF0E10)
	#define FEE0SIGN 		(*(volatile unsigned long *) 	0XFFFF0E18)
	#define FEE0PRO 		(*(volatile unsigned long *) 	0XFFFF0E1C)
	#define FEE0HID 		(*(volatile unsigned long *) 	0XFFFF0E20)
#endif
#if defined (ADUC7032) || defined (ADUC7033) || defined (ADUC7128) || defined (ADUC7129)
	#define FEE1STA 		(*(volatile unsigned char *) 	0XFFFF0E80)
	#define FEE1MOD 		(*(volatile unsigned char *)	0XFFFF0E84)
	#define FEE1CON 		(*(volatile unsigned char *) 	0XFFFF0E88)
	#define FEE1DAT 		(*(volatile unsigned short *) 	0XFFFF0E8C)
	#define FEE1ADR 		(*(volatile unsigned short *) 	0XFFFF0E90)
	#define FEE1SIGN 		(*(volatile unsigned long *) 	0XFFFF0E98)
	#define FEE1PRO 		(*(volatile unsigned long *) 	0XFFFF0E9C)
	#define FEE1HID 		(*(volatile unsigned long *) 	0XFFFF0EA0)
#endif
	
/*  PWM     */
#if defined (ADUC7019) || defined (ADUC7020) || defined (ADUC7021) || defined (ADUC7022) \
	|| defined (ADUC7023) || defined (ADUC7024) || defined (ADUC7025) || defined (ADUC7026) \
	|| defined (ADUC7027) || defined (ADUC7028)

	#define PWMCON 		(*(volatile unsigned short *) 	0XFFFFFC00)
	#define PWMSTA	 	(*(volatile unsigned short *) 	0XFFFFFC04)
	#define PWMDAT0 	(*(volatile unsigned short *) 	0XFFFFFC08)
	#define PWMDAT1 	(*(volatile unsigned short *) 	0XFFFFFC0C)
	#define PWMCFG 		(*(volatile unsigned short *) 	0XFFFFFC10)
	#define PWMCH0 		(*(volatile unsigned short *) 	0XFFFFFC14)
	#define PWMCH1 		(*(volatile unsigned short *) 	0XFFFFFC18)
	#define PWMCH2 		(*(volatile unsigned short *) 	0XFFFFFC1C)
	#define PWMEN 		(*(volatile unsigned short *) 	0XFFFFFC20)
	#define PWMDAT2 	(*(volatile unsigned short *) 	0XFFFFFC24)
#endif
#if defined (ADUC7128) || defined (ADUC7129)
	#define PWMCON1		(*(volatile unsigned short *) 	0XFFFF0F80)
	#define PWM1COM1	(*(volatile unsigned short *) 	0XFFFF0F84)
	#define PWM1COM2	(*(volatile unsigned short *) 	0XFFFF0F88)
	#define PWM1COM3	(*(volatile unsigned short *) 	0XFFFF0F8C)
	#define PWM1LEN		(*(volatile unsigned short *) 	0XFFFF0F90)
	#define PWM2COM1	(*(volatile unsigned short *) 	0XFFFF0F94)
	#define PWM2COM2	(*(volatile unsigned short *) 	0XFFFF0F98)
	#define PWM2COM3	(*(volatile unsigned short *) 	0XFFFF0F9C)
	#define PWM2LEN		(*(volatile unsigned short *) 	0XFFFF0FA0)
	#define PWM3COM1	(*(volatile unsigned short *) 	0XFFFF0FA4)
	#define PWM3COM2	(*(volatile unsigned short *) 	0XFFFF0FA8)
	#define PWM3COM3	(*(volatile unsigned short *) 	0XFFFF0FAC)
	#define PWM3LEN		(*(volatile unsigned short *) 	0XFFFF0FB0)
	#define PWMCON2		(*(volatile unsigned short *) 	0XFFFF0FB4)
	#define PWMICLR		(*(volatile unsigned short *) 	0XFFFF0FB8)
#endif

/* Quadrature Encoder */	
#if defined (ADUC7128) || defined (ADUC7129)
	#define QENCON		(*(volatile unsigned short *) 	0XFFFF0F00)
	#define QENSTA		(*(volatile unsigned char*) 	0XFFFF0F04)
	#define QENDAT		(*(volatile unsigned short *) 	0XFFFF0F08)
	#define QENVAL		(*(volatile unsigned short *) 	0XFFFF0F0C)
	#define QENCLR		(*(volatile unsigned char *) 	0XFFFF0F14)
	#define QENSET		(*(volatile unsigned char *) 	0XFFFF0F18)
#endif

/* STI Interface */
#if defined (ADUC7033) || defined (ADUC7034)
	#define STIKEY0		(*(volatile unsigned long *) 	0XFFFF0880)
	#define STICON		(*(volatile unsigned short *) 	0XFFFF0884)
	#define STIKEY1		(*(volatile unsigned long *) 	0XFFFF0888)
	#define STIDAT0		(*(volatile unsigned short *) 	0XFFFF088C)
	#define STIDAT1		(*(volatile unsigned short *) 	0XFFFF0890)
	#define STIDAT2		(*(volatile unsigned short *) 	0XFFFF0894)
#endif

/* LIN Interface */
#if defined (ADUC7032) || defined (ADUC7033) || defined (ADUC7034)
	#define LHSSTA		(*(volatile unsigned char *) 	0XFFFF0780)
	#define LHSCON0		(*(volatile unsigned short *) 	0XFFFF0784)
	#define LHSVAL0		(*(volatile unsigned short *) 	0XFFFF0788)
	#define LHSCON1		(*(volatile unsigned char *) 	0XFFFF078C)
	#define LHSVAL1		(*(volatile unsigned short *) 	0XFFFF0790)
#endif
#if defined (ADUC7033) || defined (ADUC7034)
	#define LHSCAP		(*(volatile unsigned short *) 	0XFFFF0794)
	#define LHSCMP		(*(volatile unsigned short *) 	0XFFFF0798)
#endif
	
/* High Volatage Interface */
#if defined (ADUC7033) || defined (ADUC7034)
	#define HVCON		(*(volatile unsigned char *) 	0XFFFF0804)
	#define HVDAT		(*(volatile unsigned short *) 	0XFFFF080C)
#endif
	
#endif /*ADUC_H_*/
